/* FILE NAME: client.c */
/******* Client of the inventory application with implicit method ***********/
#include <stdio.h>
#include <stdlib.h>
#include "inv.h"                 /* header file created by the IDL compiler */
#include "..\status.h"

char instructions[] = "Type character followed by appropriate argument(s).\n\
   Is part available?               a  [part_number]\n\
   What is part name?               n  [part_number]\n\
   Get part description.            d  [part_number]\n\
   What is part price?              p  [part_number]\n\
   What is part quantity?           q  [part_number]\n\
   What are subparts of this part?  s  [part_number]\n\
   Order part.                      o  part_number  quantity\n\
   REDISPLAY                        r\n\
   EXIT                             e\n";

main()
{
   part_record part;                /* structure for all data about a part  */
   part_list   *subparts;           /* pointer to parts list data structure */
   account_num account = 1234;      /* a user account number                */
   unsigned long   status;             /* error status                      */

   int i, num_args, done = 0;
   long result;
   char input[100], selection[20], quantity[20];

   puts(instructions);
   part.number = 0;
   strcpy(quantity, "");

#ifndef LOCAL                       /* find server in name service database */
   do_import_binding("", &global_binding_h);
#endif

   status = RpcBindingReset(global_binding_h);
   CHECK_STATUS(status, "Can't reset binding handle", ABORT);

   while(!done) {            /* user makes selections and each is processed */
      printf("Selection: ");  fflush(stdout);  gets(input);
      num_args = sscanf(input, "%s%ld%s", selection, &(part.number), quantity);

      switch (tolower(selection[0])) {
      case 'a': if (is_part_available(part.number))
                   puts("available: Yes");
                else
                   puts("available: No");
                break;
      case 'n': whatis_part_name(part.number, part.name);
                printf("name:%s\n", part.name);
                break;
      case 'd': part.description = get_part_description(part.number);
                printf("description:\n%s\n", part.description);
                if(part.description != NULL)
                   free(part.description);      /* free memory allocated */
                break;
      case 'p': whatis_part_price(part.number, &(part.price));
                printf("price:%10.2f\n", part.price.per_unit);
                break;
      case 'q': whatis_part_quantity(part.number, &(part.quantity));
                if(part.quantity.units == ITEM)
                   printf("total items:%ld\n", part.quantity.total.number);
                else if(part.quantity.units == GRAM)
                   printf("total grams:%10.2f\n", part.quantity.total.weight);
                else if(part.quantity.units == KILOGRAM)
                   printf("total kilos:%10.2f\n", part.quantity.total.weight);
                break;
      case 's': whatare_subparts(part.number, &subparts);
                for(i = 0; i < subparts->size; i++)
                   printf("%ld  ", subparts->numbers[i]);
                printf("\ntotal number of subparts:%ld\n", subparts->size);
                free(subparts);        /* free memory for conformant struct */
                break;
      case 'o': if(num_args < 3) {
                   puts("Not enough arguments");
                   break;
                }
                /* Assume KILOGRAM units and assign quantity input */
                part.quantity.units = KILOGRAM;
                part.quantity.total.weight = atof(quantity);
                result = order_part(part.number, &(part.quantity), account);
                if(result > 0) {
                  if(part.quantity.units == ITEM)
                    printf("order:%ld items\n", part.quantity.total.number);
                  else if(part.quantity.units == GRAM)
                    printf("order:%10.2f grams\n", part.quantity.total.weight);
                  else if(part.quantity.units == KILOGRAM)
                    printf("order:%10.2f kilos\n", part.quantity.total.weight);
                }
                else { /* error cases */
                   if(result == -1) puts("Invalid part number");
                   else if(result == -2) puts("Invalid quantity");
                   else if(result == -3) puts("Invalid account number");
                }
                break;
      case 'r':   /* redisplay selection or bad input displays instructions */
      default:  puts(instructions);  break;
      case 'e': done = 1;  break;
      } /*end case */
   } /* end while */
} /* end main() */

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );

}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}
